'use strict'
const exec = require('child_process').exec
const co = require('co')
const prompt = require('co-prompt')
const config = require('../templates')
const defaultConf = require("../default")
const chalk = require('chalk')
const getGitUser = require('./git-user')
const fs = require('fs')
const inquirer = require('inquirer')
const templates = Object.keys(config.tpl) // 所有模板名称
const branchs = function() {
  let branchs = []
  for(let i = 0; i < templates.length; i++) {
    // console.log(config.tpl[templates[i]])
    if(branchs.indexOf(config.tpl[templates[i]].branch) === -1) {
      branchs.push(config.tpl[templates[i]].branch)
    }
  }
  return branchs
}()

module.exports = () => {
  co(function*() {
    // console.log(this.process.env.EXEPATH) // 如果是Git Bash命令环境，会输出Git Bash安装路径，否则为undefined
    const user = getGitUser() // 获取git信息
    let conf = {
      tplName: null,
      branch: null,
      projectName: null,
      version: null,
      description: null
    };
    // 如果是Git Bash环境
    if(this.process.env.EXEPATH && this.process.env.EXEPATH.toString().toUpperCase().indexOf('GIT') !== -1) {
      for(let i = 0; i < templates.length; i++) {
        console.log(`${[i]}) ${templates[i]}`)
      }
      conf.tplName = getUserInput(templates, (yield prompt('Please enter a number to select the template you want to use(请选择您要使用的模板): ')))
      console.info('----------------------------------------------------')
      for(let i = 0; i < branchs.length; i++) {
        console.log(`${[i]}) ${branchs[i]}`)
      }

      conf.branch = getUserInput(branchs, (yield prompt('Please enter a number to select the branch you want to use(请选择您要使用的分支): ')))
      conf.projectName = (yield prompt('Please enter your project name(请输入您要创建的项目名称): ')) || defaultConf.name;
      conf.version = (yield prompt('Please enter your project version(请输入您要项目版本号): '))||defaultConf.version;
      conf.description = (yield prompt('Please enter your project description(入您要项目描述): ')) || defaultConf.description;
      // 如果不是Git Bash环境
    } else { 
      inquirer.prompt([{
            type: 'list',
            name: 'tplName',
            message: 'Please select a template(请选择您要使用的模板): ',
            choices: templates
          },
          {
            type: 'list',
            name: 'branch',
            message: 'Please select a branch(请选择您要使用的分支):',
            choices: branchs
          },
          {
            type: 'input',
            name: 'projectName',
            message: "Please enter your project name(请输入您的项目名称): "
          }, {
            type: 'input',
            name: 'description',
            message: "Please enter project description(请输入项目描述): "
          },
        ])
        .then(answers => {
          conf = answers
        });
     
    }
    if(!conf.tplName || conf.tplName.trim() == "") {
      for(var t in config.tpl) {
        conf.tplName = t;
        break;
      }
    } else {
      if(!config.tpl[conf.tplName]) {
        console.log(chalk.red('\n × Template does not exit!(不存在此模板)'))
        process.exit()
      }
    }

    // TODO 分支判断处理 目前不做处理
    // if (!config.tpl[tplName].branch) {
    //   console.log(chalk.red('\n × This branch does not exit!(不存在此分支)'))
    //   process.exit()
    // }

    let tpl = config.tpl[conf.tplName];
    tpl.branch = conf.branch;

    let cmdStr = `git clone -b ${tpl.branch} ${tpl.url} ${conf.projectName}`

    console.log(cmdStr)
    console.log(chalk.white('\n Start generating...'))
    exec(cmdStr, (error, stdout, stderr) => {
      if(error) {
        console.log(error)
        process.exit()
      }
      fs.readFile(conf.projectName + '/package.json', 'utf-8', function(err, data) {
        if(err) {
          throw err;
        }
        // 写入作者信息
        // TODO描述信息暂时不做处理
        let config = JSON.parse(data)
        config.author = user;
        config.version = conf.version;
        config.description = conf.description;
        config.name = conf.projectName;
        config.private = true;
        fs.writeFile(conf.projectName + '/package.json', JSON.stringify(config, null, 2), 'utf-8', (err) => {
          if(err) console.log(err)
          console.log(chalk.green('\n √ Generation completed!'))
          console.log(`\n cd ${conf.projectName} && npm install \n`)
          process.exit()
        })
      });
      deleteFolderRecursive(conf.projectName + '/.git');

    })
  })
}

function deleteFolderRecursive(path) {
  if(fs.existsSync(path)) {
    fs.readdirSync(path).forEach(function(file) {
      var curPath = path + "/" + file;
      if(fs.statSync(curPath).isDirectory()) { // recurse
        deleteFolderRecursive(curPath);
      } else { // delete file
        fs.unlinkSync(curPath);
      }
    });
    fs.rmdirSync(path);
  }
};

function getUserInput (arrays, input) {
  if(!input) input = 0                 // 如果用户没有输入
  if(arrays[input]) return arrays[input]
  // if(arrays.indexOf(input) !== -1) return input
  return input
}